package com.onetouch;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

// 外部类库，用于加密
import org.apache.commons.codec.digest.DigestUtils;
import org.json.JSONObject;

/**
 * 一键呼叫
 */
public class OneTouch {
	// 网站的appkey
	private String appkey = "";
	// appkey 对应的秘钥
	private String secret = "";
	public String apiurl = "http://api-voicecall.mixcom.cn";

	/**
	 * 设置appkey
	 * @param appkey
	 */
	public void setAppkey(String appkey) {
		this.appkey = appkey;
	}

	/**
	 * 设置秘钥
	 * @param secret
	 */
	public void setSecret(String secret) {
		this.secret = secret;
	}

	/**
	 * 点击呼叫
	 * callerNbr 主叫号码,
	 * calleeNbr 被叫号码,
	 * maxduration 最大通话时长，单位(分钟), 如果接口没有设置， 则使用号码设置的最大通话时长，默认值为 0, 表示不限制,可选
     * Uuid 自定义的唯一 id, 允许是数字, 字母和下划线, 中划线, + = /, 支持 base64, 可选
	 * @return
	 */
	public JSONObject dial(String callerNbr, String calleeNbr, int maxduration, String Uuid)
	{
		String url = this.apiurl + "/voice/dial";
		Map<String, String> map = new HashMap<String, String>();
		map.put("callerNbr", callerNbr);
		map.put("calleeNbr", calleeNbr);
		map.put("maxduration", Integer.toString(maxduration));
		map.put("uuid", Uuid);
	    return this.sendpost(url, map);
	}

	/**
     * 通话记录列表
     * timestart 查询的开始时间, 格式如下： 1591242125, 可选
     * timeend 查询的结束时间, 格式如下: 1591242199, 可选
     * page 需要查询的页数，每页100条数据, 可选
	 */
	public JSONObject cdrLists(int timestart, int timeend, int page)
	{
		String url = this.apiurl + "/cdr/lists";
		Map<String, String> map = new HashMap<String, String>();
		map.put("timestart", Integer.toString(timestart));
		map.put("timeend", Integer.toString(timeend));
		map.put("page", Integer.toString(page));
		return this.sendpost(url, map);
	}

	/**
	 * 计费数据列表(详细账单)
     * timestart 查询的开始时间, 格式如下： 1591242125, 可选
     * timeend 查询的结束时间, 格式如下: 1591242199, 可选
     * page 需要查询的页数，每页100条数据, 可选
	 */
	public JSONObject billingLists(int timestart, int timeend, int page)
	{
		String url = this.apiurl + "/billing/lists";
		Map<String, String> map = new HashMap<String, String>();
		map.put("timestart", Integer.toString(timestart));
		map.put("timeend", Integer.toString(timeend));
		map.put("page", Integer.toString(page));
		return this.sendpost(url, map);
	}

	/**
	 * 日结报表数据
     * daystart 查询的开始日期, 格式如下： 2020-06-01, 可选
     * dayend 查询的结束日期, 格式如下: 2020-07-01, 可选
     * page 需要查询的页数，每页100条数据, 可选
	 * @return
	 */
	public JSONObject reportDay(String daystart, String dayend, int page)
	{
		String url = this.apiurl + "/report/day";
		Map<String, String> map = new HashMap<String, String>();
		map.put("daystart", daystart);
		map.put("dayend", dayend);
		map.put("page", Integer.toString(page));
		return this.sendpost(url, map);
	}

	/**
     * 月结报表数据
     * @param year 格式如下： 2020,可选
     * @return
     */
	public JSONObject reportMonth(String year)
	{
		String url = this.apiurl + "/report/month";
		Map<String, String> map = new HashMap<String, String>();
		map.put("year", year);
		return this.sendpost(url, map);
	}

	/**
	 * 查询账户余额
	 * @return
	 */
	public JSONObject balanceGet()
	{
		String url = this.apiurl + "/balance/get";
		Map<String, String> map = new HashMap<String, String>();
		return this.sendpost(url, map);
	}

	/**
	 * 充值扣款记录
     * timestart 查询的开始时间, 格式如下： 1591242125, 可选
     * timeend 查询的结束时间, 格式如下: 1591242199, 可选
     * page 需要查询的页数，每页100条数据, 可选
	 * @return
	 */
	public JSONObject rechargeLists(int timestart, int timeend, int page)
	{
		String url = this.apiurl + "/recharge/lists";
		Map<String, String> map = new HashMap<String, String>();
		map.put("timestart", Integer.toString(timestart));
		map.put("timeend", Integer.toString(timeend));
		map.put("page", Integer.toString(page));
		return this.sendpost(url, map);
	}


	/**
	 * 加入公用的参数发送请求
	 */
	private JSONObject sendpost(String url, Map<String, String> map)
	{
		map.put("time", Long.toString(System.currentTimeMillis()/ 1000) );
		map.put("appkey", this.appkey);
		String sign = this.getSign(map);
		map.put("sign", sign);
		String json = this.post(url, map);
		JSONObject obj =  new JSONObject(json);
	    return obj;
	}

	/**
	 * 发起get请求
	 */
	private String get(String url)
	{
		String resault = "";
		String line = "";
		try {
			String method = "GET";
			HttpURLConnection  conn = this.getConnection(url, method);
			conn.setReadTimeout(5000);
	        conn.setConnectTimeout(5000);
			conn.connect();
			// 获取数据
			BufferedReader read = new BufferedReader(new InputStreamReader(conn.getInputStream(), "utf-8"));
			while((line = read.readLine()) != null) {
				resault += line.toString();
			}
		} catch(Exception e) {
			e.printStackTrace();
		}
		return resault;
	}

	/**
	 * 发起post请求
	 */
	private String post(String url, Map<String, String> params)
	{
		String resault = "";
		String line = "";
		String poststr = "";
		DataOutputStream out = null;
		try {
			String method = "POST";
			HttpURLConnection  conn = this.getConnection(url, method);
			conn.setReadTimeout(5000);
	        conn.setConnectTimeout(5000);
			conn.connect();
			//处理map数据
			for(Map.Entry<String, String> entry : params.entrySet()) {
				poststr += entry.getKey() + "=" + URLEncoder.encode(entry.getValue(), "utf-8") + "&";
			}
			poststr = poststr.substring(0, poststr.length() -1);
			// post请求
			out = new DataOutputStream(conn.getOutputStream());
			out.writeBytes(poststr);
			out.flush();
			out.close();
			// 获取数据
			BufferedReader read = new BufferedReader(new InputStreamReader(conn.getInputStream(), "utf-8"));
			while((line = read.readLine()) != null) {
				line = new String(line.getBytes(), "utf-8");
				resault += line.toString();
			}
			read.close();
			conn.disconnect();
		} catch(Exception e) {
			e.printStackTrace();
		}
		return resault;
	}

	/**
	 * 获取连接
	 */
	private HttpURLConnection getConnection(String url, String method)
	{
		HttpURLConnection  connection = null;
		try {
			URL httpurl = new URL(url);
			connection = (HttpURLConnection)httpurl.openConnection();
			if(method == "GET") {
				connection.setDoOutput(false);
			} else {
				connection.setDoOutput(true);
			}
			connection.setDoInput(true);
			connection.setUseCaches(false);
			connection.setRequestMethod(method);
			connection.setInstanceFollowRedirects(true);
			connection.setRequestProperty("content-type", "application/x-www-form-urlencoded");
			connection.setRequestProperty("Charset", "utf-8");
			connection.setRequestProperty("Accept-Charset", "utf-8");
		} catch(Exception e) {
			e.printStackTrace();
		}
		return connection;
	}

	/**
	 * 生成加密检验码
	 * @param params
	 * @return
	 */
	private String getSign(Map<String, String> params) {
		Iterator<Map.Entry<String, String>> it = params.entrySet().iterator();
		while(it.hasNext()) {
			Map.Entry<String, String> entry = it.next();
			if(entry.getValue() == "") {
				it.remove();
			}
			if(entry.getKey() == "sign") {
				it.remove();
			}
			if(entry.getKey() == "a") {
				// 加密的时候a会排除，请勿使用
				it.remove();
			}
			if(entry.getKey() == "c") {
				// 加密的时候c会排除，请勿使用
				it.remove();
			}
		}

		// map排序
		List<Map.Entry<String, String>> list = new ArrayList<Map.Entry<String, String>>(params.entrySet());
		Collections.sort(list, new Comparator<Map.Entry<String, String>>(){
			public int compare(Map.Entry<String, String> s1, Map.Entry<String, String> s2) {
				return s1.getKey().compareTo(s2.getKey());
			}
		});
		// 生成sign
		String str = new String();
		for(Map.Entry<String,String> mapping:list){
			str += mapping.getValue();
        }
		str += this.secret;
		// 需要导入外部的类库
		// http://commons.apache.org/proper/commons-codec/
		// http://search.maven.org/remotecontent?filepath=commons-codec/commons-codec/1.11/commons-codec-1.11.jar
		// 如果sha1Hex无法使用，请尝试使用shaHex. shaHex为已过期的方法，推荐使用sha1Hex
		String sign = DigestUtils.sha1Hex(str + DigestUtils.sha1Hex(this.secret)).toLowerCase();
		return sign;
	}
}
